@@ -426,10 +426,12 @@ class DicomFile(Data):
         self.path = fullpath(path)
 
         # Attempt to read DICOM dataset.
-        try:
-            self.ds = pydicom.dcmread(self.path, force=True)
-        except IsADirectoryError:
-            self.ds = None
+        self.ds = None
+        if not os.path.isdir(self.path):
+            try:
+                self.ds = pydicom.dcmread(self.path, force=True)
+            except IsADirectoryError:
+                pass
 
         # Try to protect against cases where file read isn't a DICOM file.
         if (not isinstance(self.ds, pydicom.dataset.FileDataset)
@@ -2108,3 +2110,16 @@ def get_dict_permutations(in_dict=None):
     else:
         permutations = [{}]
     return permutations
+
+def qualified__name(cls):
+    """
+    Return qualified name of a class.
+
+    **Parameter:**
+
+    cls : class
+        Class for which qualified name is to be determined.  If non-class
+        is passed as argument, None is returned.
+    """
+    if isinstance(cls, type):
+        return f"{cls.__module__}.{cls.__name__}"
