@@ -1,5 +1,50 @@
-"""Tools for performing image segmentation."""
+"""
+Classes and functions relating to image segmentation.
+
+This module implements atlas-based segmentation, where an atlas is
+a reference image, for which regions of interest (ROIs) have been segmented
+in some way.
+
+In single-atlas segmentation, a single atlas is registered to a target image
+that is to be segmented.  Atlas segmentations are then mapped to the target
+image, using the registration transform.  An ROI may be represented as
+a binary mask or as sets of contour points, resulting in two strategies
+for mapping ROIs:
+
+    - atlas and target are taken as fixed and moving image respectively for
+      registration, then ROI contour points are pushed from atlas to target;
+    - atlas and target are taken as moving and fixed image respectively for
+      the registration, then ROI binary masks are pulled from atlas to target.
+
+In multi-atlas segmentation, single-atlas segmentation is performed
+multiple times, using a different atlas each time, then the results from
+the individual segmentations are combined in some way.
+
+The following classes are defined:
+SingleAtlasSegmentation() : class for performing single-atlas segmentation;
+MultiAtlasSegmentation() : class for performing multi-atlas segmentation.
+
+The following are utility functions used by SingleAtlasSegmentation()
+and MultiAtlasSegmentation():
+ensure_any()
+ensure_dict()
+ensure_image()
+ensure_structure_set()
+get_contour_propagation_strategies()
+get_fixed_and_moving()
+get_option()
+get_options()
+get_segmentation_steps()
+get_steps()
+get_structure_set_index()
+
+The following function allows for comparison of different single-atlas
+segmentations:
+get_sas_comparisons()
+"""
+
 from concurrent.futures import ThreadPoolExecutor
+from copy import deepcopy
 from inspect import signature
 from itertools import combinations
 from pathlib import Path
@@ -23,7 +68,7 @@ class MultiAtlasSegmentation(Data):
         auto=False, auto_step=None, auto_strategies=None,
         auto_reg_setup_only=False,
         default_step=None, default_strategy="pull", roi_names=None,
-        metrics=None, slice_stats=None, default_by_slice=None,
+        metrics=None, default_slice_stats=None, default_by_slice=None,
         consensus_types="majority", default_consensus_type="majority",
         max_workers=1, **kwargs):
 
@@ -67,7 +112,9 @@ class MultiAtlasSegmentation(Data):
 
         # Set parameters for post-segmentation ROI comparisons.
         self.metrics = metrics
-        self.slice_stats = slice_stats
+        self.default_slice_stats = (Defaults().slice_stats
+                                 if default_slice_stats is None
+                                 else default_slice_stats)
         self.default_by_slice = (Defaults().by_slice
                                  if default_by_slice is None
                                  else default_by_slice)
@@ -247,6 +294,9 @@ class MultiAtlasSegmentation(Data):
             name_as_index=False, atlas_ids_to_compare=False,
             combination_length=None, **kwargs):
 
+        if not hasattr(self, "ss1_filtered"):
+            return
+
         consensus_types = get_options(
                 consensus_types, self.consensus_types, get_consensus_types())
         if not (is_list(atlas_ids) and atlas_ids and is_list(atlas_ids[0])):
@@ -268,8 +318,7 @@ class MultiAtlasSegmentation(Data):
         steps = get_options(
                 steps, self.default_step, self.steps)
         metrics = get_options(metrics, self.metrics, get_comparison_metrics())
-        slice_stats = get_options(slice_stats, self.slice_stats,
-                                  get_stat_functions())
+        slice_stats = slice_stats or self.default_slice_stats
         default_by_slice = get_option(default_by_slice, self.default_by_slice,
                                       get_by_slice_methods())
 
@@ -282,6 +331,7 @@ class MultiAtlasSegmentation(Data):
         
         df = None
         ss1 = sas.ss1_filtered.filtered_copy(to_keep=to_keep)
+        ss1.set_image(self.im1, add_to_image=False)
         for idx, id2 in enumerate(atlas_ids_to_compare + consensus_types):
             if idx < n_atlas_ids_to_compare:
                 sas = self.get_sas(id2)
@@ -354,7 +404,7 @@ class MultiAtlasSegmentation(Data):
             scores1[score].append(atlas_id)
 
         scores2 = {}
-        for score, atlas_ids in sorted(scores1.items()):
+        for score, atlas_ids in sorted(scores1.items(), reverse=True):
             for atlas_id in sorted(atlas_ids):
                 scores2[atlas_id] = score
         
@@ -379,7 +429,7 @@ class SingleAtlasSegmentation(Data):
         roi_pfiles=None, crop_to_match_size2=True, voxel_size2=None,
         default_roi_bands=None, roi_bands=None, pfiles2=None,
         most_points2=True, capture_output=False,
-        keep_tmp_dir=False, metrics=None, slice_stats=None,
+        keep_tmp_dir=False, metrics=None, default_slice_stats=None,
         default_by_slice=None):
 
         # Set images and associated structure sets.
@@ -449,7 +499,9 @@ class SingleAtlasSegmentation(Data):
 
         # Set parameters for post-segmentation ROI comparisons.
         self.metrics = metrics
-        self.slice_stats = slice_stats
+        self.default_slice_stats = (Defaults().slice_stats
+                                 if default_slice_stats is None
+                                 else default_slice_stats)
         self.default_by_slice = (Defaults().by_slice
                                  if default_by_slice is None
                                  else default_by_slice)
@@ -641,6 +693,11 @@ class SingleAtlasSegmentation(Data):
         else:
             return self.registrations[strategy][step]
 
+    def get_mutual_information(self, strategy=None, step=None, roi_name=None,
+            reg_step=None, force=False, **kwargs):
+        reg = self.get_registration(strategy, step, roi_name, force)
+        return reg.get_mutual_information(reg_step, **kwargs)
+
     def get_segmentation(self, strategy=None, step=None, reg_step=None,
             force=False):
         strategy = get_option(strategy, self.default_strategy, self.strategies)
@@ -698,21 +755,46 @@ class SingleAtlasSegmentation(Data):
             self, id1=None, id2=None, to_keep=None, strategies=None,
             steps=None, reg_steps=None, force=False, metrics=None,
             slice_stats=None, default_by_slice=None, 
-            voxel_size=None, name_as_index=False, **kwargs):
+            voxel_size=None, name_as_index=False,
+            mi_variants=None, mi_init=False, mi_reg_step=False, **kwargs):
+
+        if not hasattr(self, "ss1_filtered"):
+            return
 
         strategies = get_options(
                 strategies, self.default_strategy, self.strategies)
         steps = get_options(
                 steps, self.default_step, self.steps)
         metrics = get_options(metrics, self.metrics, get_comparison_metrics())
-        slice_stats = get_options(slice_stats, self.slice_stats,
-                                  get_stat_functions())
+        slice_stats = slice_stats or self.default_slice_stats
         default_by_slice = get_option(default_by_slice, self.default_by_slice,
                                       get_by_slice_methods())
-        
+
+        if mi_variants is None:
+            mi_variants = ["mi"]
+        elif isinstance(mi_variants, str):
+            mi_variants = [mi_variants]
+        elif not mi_variants:
+            mi_variants = []
+
+        mi_parameters = list(signature(Image.get_mutual_information).parameters)
+        comparison_parameters = list(signature(
+            SingleAtlasSegmentation.get_comparison).parameters)
+        mi_kwargs = {}
+        comparison_kwargs = {}
+        for key, value in kwargs.items():
+            if key in mi_parameters:
+                mi_kwargs[key] = value
+            else:
+                comparison_kwargs[key] = value
+
         df = None
         ss1 = self.ss1_filtered.filtered_copy(to_keep=to_keep)
+        ss1.set_image(self.im1, add_to_image=False)
         for strategy in strategies:
+            method = kwargs.get("method", None)
+            if method is None:
+                kwargs["method"] = "mask" if "pull" == strategy else "contour"
             for step in steps:
                 self.segment(strategy, step, force)
                 all_reg_steps = list(self.segmentations[strategy][step])
@@ -720,11 +802,12 @@ class SingleAtlasSegmentation(Data):
                 for reg_step in step_reg_steps:
                     ss2 = (self.get_segmentation(strategy, step, reg_step)
                            .filtered_copy(to_keep=to_keep))
+
                     df_tmp = ss1.get_comparison(
                             ss2, metrics=metrics, slice_stats=slice_stats,
                             default_by_slice=default_by_slice,
                             voxel_size=voxel_size, name_as_index=name_as_index,
-                            **kwargs)
+                            **comparison_kwargs)
 
                     if df_tmp is None:
                         continue
@@ -736,6 +819,23 @@ class SingleAtlasSegmentation(Data):
                             df_tmp[label] = pd.Series(
                                     df_tmp.shape[0] * [value])
 
+                    if mi_init:
+                        for mi_variant in mi_variants:
+                            mi_kwargs["variant"] = mi_variant
+                            mi = self.get_mutual_information(
+                                    strategy, 0, None, 0, **mi_kwargs)
+                            df_tmp[f"{mi_variant}_init"] = pd.Series(
+                                    df_tmp.shape[0] * [mi])
+
+                    if mi_reg_step:
+                        for mi_variant in mi_variants:
+                            mi_kwargs["variant"] = mi_variant
+                            #print(df_tmp.apply(lambda row: print(row), axis=1))
+                            df_tmp[f"{mi_variant}_reg_step"] = df_tmp.apply(
+                                    lambda row: self.get_mutual_information(
+                                        strategy, step, row["ROI"], reg_step,
+                                        **mi_kwargs), axis=1)
+
                     if df is None:
                         df = df_tmp
                     else:
@@ -880,82 +980,99 @@ def get_structure_set_index(ss_index, im):
         return len(im.structure_sets) + ss_index
     return ss_index
 
-def get_sas_comparisons(pfiles1_variations=None, pfiles2_variations=None,
-                        **kwargs):
 
-    if not (pfiles1_variations or pfiles2_variations or kwargs):
-        return
+class SasTuner(Data):
 
-    sas_constant_kwargs = {}
-    sas_variable_kwargs = {}
-    comparison_kwargs = {}
+    def __init__(self, pfiles1_variations=None, pfiles2_variations=None,
+                 keep_segmentations=False, **kwargs):
 
-    sas_parameters = list(signature(SingleAtlasSegmentation).parameters)
+        self.pfiles1_variations = pfiles1_variations
+        self.pfiles2_variations = pfiles2_variations
+        self.keep_segmentations = keep_segmentations
+        self.kwargs = kwargs
 
-    for key, value in kwargs.items():
-        if key in sas_parameters:
-            if isinstance(value, list) and len(value) > 1:
-                sas_variable_kwargs[key] = value
-            else:
-                sas_constant_kwargs[key] = value
+        self.sas_constant_kwargs = {}
+        self.sas_variable_kwargs = {}
+        self.comparison_kwargs = {}
+
+        if kwargs:
+            sas_parameters = list(signature(SingleAtlasSegmentation).parameters)
+            comparison_parameters = list(signature(
+                SingleAtlasSegmentation.get_comparison).parameters)
+            for key, value in kwargs.items():
+                if key in sas_parameters and key not in comparison_parameters:
+                    if isinstance(value, list) and len(value) > 1:
+                        self.sas_variable_kwargs[key] = value
+                    else:
+                        self.sas_constant_kwargs[key] = value
+                else:
+                    self.comparison_kwargs[key] = value
+
+            self.sas_constant_kwargs["auto"] = True
+            self.sas_constant_kwargs["auto_reg_setup_only"] = True
+            self.sas_constant_kwargs["overwrite"] = True
+
+        if pfiles1_variations:
+            reg1_permutations = get_dict_permutations(
+                    {reg_step:
+                     get_dict_permutations(pfiles1_variations[reg_step])
+                     for reg_step in pfiles1_variations})
         else:
-            comparison_kwargs[key] = value
+            reg1_permutations = [{}]
 
-    sas_constant_kwargs["auto"] = True
-    sas_constant_kwargs["auto_reg_setup_only"] = True
-    sas_constant_kwargs["overwrite"] = True
+        if pfiles2_variations:
+            reg2_permutations = get_dict_permutations(
+                    {reg_step:
+                     get_dict_permutations(pfiles2_variations[reg_step])
+                     for reg_step in pfiles2_variations})
+        else:
+            reg2_permutations = [{}]
 
-    if pfiles1_variations:
-        reg1_permutations = get_dict_permutations(
-                {reg_step: get_dict_permutations(pfiles1_variations[reg_step])
-                 for reg_step in pfiles1_variations})
-    else:
-        reg1_permutations = [{}]
+        sas_permutations = get_dict_permutations(self.sas_variable_kwargs)
 
-    if pfiles2_variations:
-        reg2_permutations = get_dict_permutations(
-                {reg_step: get_dict_permutations(pfiles2_variations[reg_step])
-                 for reg_step in pfiles2_variations})
-    else:
-        reg2_permutations = [{}]
+        self.df = None
+        self.sass = []
+        self.adjustments = []
 
-    sas_permutations = get_dict_permutations(sas_variable_kwargs)
+        for sas_permutation in sas_permutations:
+            sas = SingleAtlasSegmentation(
+                    **self.sas_constant_kwargs, **sas_permutation)
+            strategies = get_options(
+                    sas.auto_strategies, sas.default_strategy, sas.strategies)
+            for reg1_permutation in reg1_permutations:
+                reg1_adjustments = sas.adjust_reg_files(
+                        strategies, sas.steps[0], None, reg1_permutation)
+                for reg2_permutation in reg2_permutations:
+                    reg2_adjustments = sas.adjust_reg_files(
+                            strategies, sas.steps[1],
+                            sas.roi_names, reg2_permutation)
+                    adjustments = {**sas_permutation, **reg1_adjustments,
+                                   **reg2_adjustments}
 
-    df = None
-    for sas_permutation in sas_permutations:
-        sas = SingleAtlasSegmentation(**sas_constant_kwargs, **sas_permutation)
-        strategies = get_options(
-                sas.auto_strategies, sas.default_strategy, sas.strategies)
-        for reg1_permutation in reg1_permutations:
-            reg1_adjustments = sas.adjust_reg_files(
-                    strategies, sas.steps[0], None, reg1_permutation)
-            for reg2_permutation in reg2_permutations:
-                reg2_adjustments = sas.adjust_reg_files(
-                        strategies, sas.steps[1],
-                        sas.roi_names, reg2_permutation)
-                adjustments = {**sas_permutation, **reg1_adjustments,
-                               **reg2_adjustments}
+                    df_comparison = sas.get_comparison(**self.comparison_kwargs)
 
-                df_comparison = sas.get_comparison(**comparison_kwargs)
+                    self.adjustments.append(deepcopy(adjustments))
+                    if keep_segmentations:
+                        self.sass.append(sas.clone())
 
-                if adjustments:
-                    df_permutation = pd.DataFrame(
-                            max(1, df_comparison.shape[0]) * [adjustments])
-                else:
-                    df_permutation = None
+                    if adjustments:
+                        df_permutation = pd.DataFrame(
+                                max(1, df_comparison.shape[0]) * [adjustments])
+                    else:
+                        df_permutation = None
 
-                if df_permutation is None and df_comparison is None:
-                    continue
-                elif df_permutation is None:
-                    df_sas = df_comparison
-                elif df_comparison is None:
-                    df_sas = df_permutation
-                else:
-                    df_sas = pd.concat([df_permutation, df_comparison], axis=1)
+                    if df_permutation is None and df_comparison is None:
+                        continue
+                    elif df_permutation is None:
+                        df_sas = df_comparison
+                    elif df_comparison is None:
+                        df_sas = df_permutation
+                    else:
+                        df_sas = pd.concat(
+                                [df_permutation, df_comparison], axis=1)
 
-                if df is None:
-                    df = df_sas
-                else:
-                    df = pd.concat([df, df_sas], ignore_index=True)
-
-    return df
+                    if self.df is None:
+                        self.df = df_sas
+                    else:
+                        self.df = pd.concat(
+                                [self.df, df_sas], ignore_index=True)
